<template>
	<div class='comments-widget-wrapper' :style='widgetWrapperStyle'>
		<div class="comments-widget" @mouseenter='cancelScroll' @mouseleave='scrollToBottom'>
			<div class='comments-wrapper' ref='commentsWrapper'>
				<div class='comments'>
					<div
						class='comment'
						:key='comment.id'
						v-for='(comment, index) in displayComments'
						:class='{last: index === 0}'
					>
						<span
							class='name'
							:style='getNameStyle(comment.name)'
							@click='insertReply(comment.name)'
						>
							{{comment.name}}:
						</span>
						<span class='text' v-html='comment.text'></span>

						<div v-if='isAdmin' class='admin-actions'>
							<div class='delete' @click='deleteComment(comment.id)'>❌</div>
							<div class='ban' @click='banUser(comment.name)'>🟥</div>
						</div>
					</div>
				</div>
			</div>
			<div class='form' @click='openAuthForm' @focus='openAuthForm'>
				<label class='sendbox' @keydown.enter='sendComment'>
					<input
						ref='commentInput'
						class='text'
						v-model='text'
						placeholder='Комментарий'
						maxlength='450'
					/>

					<span class='timer' :class='{visible: countdownTime}'>
						{{ countdownTime }}
					</span>
				</label>
				<p v-if="errorMessage" class="error">{{ errorMessage }}</p>
			</div>
		</div>
	</div>
</template>


<style scoped>

.comments-widget {
	color: white;
	font-family: var(--primary-font);
	font-variation-settings: 'wght' 350;
	height: 100%;
	padding: 0 .3em .45em .7em;

	display: flex;
	flex-direction: column;

	width: 10em;
	pointer-events: all;

	position: absolute;
	right: 0;
	bottom: 0;

	transition: background .5s ease, backdrop-filter .5s ease;
}

.comments-widget:hover, .comments-widget.hovered {
	background: rgba(0, 0, 0, .85);
}

.comments-widget:hover .comment:not(.last), .comments-widget.hovered .comment:not(.last) {
	opacity: 1;
}

.comments-widget:hover .form input, .comments-widget.hovered .form input {
	background: none;
	border-color: #333;
}

.comments-wrapper {
	flex-grow: 1;
	scrollbar-width: none;
	overflow-y: scroll;
	overflow-x: hidden;
	display: flex;
	flex-direction: column-reverse;
}

.comments-wrapper::-webkit-scrollbar {
	display: none;
}

.comments {
	display: flex;
	flex-direction: column-reverse;
	gap: .1em;
	padding-bottom: .2em;

	font-size: .6em;
	text-shadow: rgba(0, 0, 0, .2) .1em .1em 0;

	border-radius: .5em;
}

.comment {
	line-height: 1.2;
	padding: .3em .5em .3em 0;
	position: relative;
	word-break: break-word;
}

.comment:first-child {
	padding-bottom: .15em;
}

.comment:not(.last) {
	opacity: 0;
}

.admin-actions {
	position: absolute;
	right: .5em;
	top: .3em;

	cursor: pointer;

	opacity: 0;
	transition: opacity .2s ease;
	font-variation-settings: 'wght' 700;

	display: flex;
	align-items: center;
	gap: .5rem;
}

.comment:hover .admin-actions {
	opacity: 1;
}

.name {
	font-variation-settings: 'wght' 800;
	margin-right: .3em;
	cursor: pointer;
}

.text >>> .highlight {
	font-variation-settings: 'wght' 500;
	text-shadow: none;
	padding: .1em .3em .15em .3em;
	margin-top: .1em;
	border-radius: .3em;
	color: black;
}

.text >>> a {
	color: #434de9;
	text-decoration: underline;
	text-decoration-thickness: 0.05em;
	text-underline-offset: 0.13em;
	text-decoration-skip-ink: none;
	transition: .3s;
}

.text >>> a:hover {
	color: red !important;
}

.form {
	font-size: .6em;
	font-variation-settings: 'wght' 400;
	position: relative;
}

.form .error {
    background-color: black;
    border-radius: .8rem;
    color: red;
    font-size: 1.2rem;
    left: -12px;
    padding: .3rem .8rem;
    position: absolute;
	top: 2px;
}

.timer {
	display: inline-block;
	width: 3ch;
	text-align: center;

	position: absolute;
	top: 50%;
	transform: translateY(-50%);
	left: 70%;

	opacity: 0;
	transition: opacity .5s;
}

.timer.visible {
	opacity: .6;
}

.form input {
	width: 100%;

	background: rgba(0, 0, 0, 0.05);
	backdrop-filter: blur(20px);

	border-radius: .5em;
	border: .1em solid transparent;

	padding: .5em 3.7em .5em .5em;
	margin-left: -.5em;

	outline: none;
	color: white;
	opacity: 1;

	transition: background .5s ease, border .5s ease;
}

.comments-widget .form input::placeholder {
	color: rgba(255, 255, 255, 0.6);
}

.sendbox {
	position: relative;
	margin-top: .1em;

	display: flex;
	align-items: center;
}

.send {
	position: absolute;
	right: .5em;
	top: .55em;
	margin-left: .5em;
	cursor: pointer;
}

.send img {
	height: 1.4em;
	filter: invert(100%);
	opacity: .6;
}

</style>


<style>
.comment {
	transition: transform .5s, opacity .5s;
}
</style>


<script>
import send from './send.svg'
import {widgetMixin, positionMixin} from '../helpers/widgets.mjs'
import {styleMixin} from '../helpers/style.mjs'


function getCookie(name) {
	const matches = document.cookie.match(new RegExp(
		'(?:^|; )' + name.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, '\\$1') + '=([^;]*)',
	))
	return matches ? decodeURIComponent(matches[1]) : undefined
}

export default {
	mixins: [widgetMixin, positionMixin, styleMixin],

	data() {
		return {
			admins: [],
			comments: [],
			countdownTime: 0,
			errorMessage: '',
			sb: [],
			scrollTimeout: null,
			send,
			text: '',
			timeout: 10,
		}
	},

	computed: {
		login() {
			return window.alsLoginUtils.getLogin()
		},

		color() {
			return this.getNameStyle(this.login).color
		},

		isAdmin() {
			return this.admins.includes(this.login)
		},

		displayComments() {
			const comments = this.comments.concat(this.sb).sort((a, b) => (b.timestamp - a.timestamp))
			const out = []

			for (const comment of comments) {
				// const regex = `@${this.login}`
				// const text = escapeHtml(comment.text)
				const text = comment.text

				// text = text.replaceAll(regex, `<span class="highlight" style="background-color: ${this.color.replace('50%', '40%')}">${regex}</span>`)

				out.push({
					...comment,
					text,
				})
			}

			return out
		},
	},

	created() {
		this.getComments()
		this.sb = JSON.parse(localStorage.getItem('comments_sb') ?? '[]')

		this.commentsInterval = setInterval(this.getComments, 5000)
	},

	beforeUnmount() {
		clearInterval(this.commentsInterval)
		clearInterval(this.countdownInterval)
	},

	watch: {
		sb(value) {
			console.log('sb(value) {: ', value)
			console.log('JSON.stringify(value)sb(value) {: ', JSON.stringify(value))
			localStorage.setItem('comments_sb', JSON.stringify(value))
		},
	},

	methods: {
		validateText(inputText) {
			// Регулярное выражение для поиска опасных тегов, включая незакрытые теги
			const maliciousPattern = new RegExp(
				[
					'<script\\b[^<]*>|<script\\b[^<]*(?:(?!<\\/script>)<[^<]*)*<\\/script>',
					'<style\\b[^<]*>|<style\\b[^<]*(?:(?!<\\/style>)<[^<]*)*<\\/style>',
					'<iframe\\b[^<]*>|<iframe\\b[^<]*(?:(?!<\\/iframe>)<[^<]*)*<\\/iframe>',
					'<object\\b[^<]*>|<object\\b[^<]*(?:(?!<\\/object>)<[^<]*)*<\\/object>',
					'<embed\\b[^<]*>|<embed\\b[^<]*(?:(?!<\\/embed>)<[^<]*)*<\\/embed>',
					'<form\\b[^<]*>|<form\\b[^<]*(?:(?!<\\/form>)<[^<]*)*<\\/form>',
					'<a\\b[^<]*>|<a\\b[^<]*(?:(?!<\\/a>)<[^<]*)*<\\/a>',
					'<meta\\b[^<]*>',
					'<link\\b[^<]*>',
					'<base\\b[^<]*>',
					'<applet\\b[^<]*>|<applet\\b[^<]*(?:(?!<\\/applet>)<[^<]*)*<\\/applet>',
					'<video\\b[^<]*>|<video\\b[^<]*(?:(?!<\\/video>)<[^<]*)*<\\/video>',
					'<audio\\b[^<]*>|<audio\\b[^<]*(?:(?!<\\/audio>)<[^<]*)*<\\/audio>',
					'<source\\b[^<]*>|<source\\b[^<]*(?:(?!<\\/source>)<[^<]*)*<\\/source>',
					'<noscript\\b[^<]*>|<noscript\\b[^<]*(?:(?!<\\/noscript>)<[^<]*)*<\\/noscript>',
				].join('|'),
				'gi',
			)

			// Проверяем текст на соответствие паттерну
			return maliciousPattern.test(inputText)
		},
		sendComment() {
			if (!this.text) return
			if (!this.login) return this.openAuthForm()
			if (this.countdownTime) return
			if (this.validateText(this.text)) {
				// Если текст в инпуте чата содержит вредоносный код, отображаем сообщение об ошибке
				this.errorMessage = 'Ваше сообщение содержит недопустимый контент!'
				this.text = ''
				this.$refs.commentInput.value = ''

				setTimeout(() => {
					this.errorMessage = ''
				}, 4000)

				return
			} else {
				this.errorMessage = ''
			}

			this.act('comment', {auth: this.getAuth(), text: this.text}).then(
				(response) => {
					if (response.data.comments) {
						this.comments = response.data.comments
						this.admins = response.data.admins
					}

					if (response.data.sb) {
						this.sb.push(response.data)
					}

					clearInterval(this.countdownInterval)

					this.countdownTime = this.timeout

					this.countdownInterval = setInterval(() => {
						this.countdownTime -= 1
						if (this.countdownTime < 0) this.countdownTime = 0
					}, 1000)
				},
			).catch((error) => {
				console.error('sendComment error: ', error)
			})

			this.text = ''
		},

		deleteComment(commentId) {
			this.act('delete_comment', {auth: this.getAuth(), comment_id: commentId}).then(
				(response) => {
					this.comments = response.data.comments
				},
			)
		},

		banUser(username) {
			this.act('ban_user', {auth: this.getAuth(), username}).then(
				(response) => {
					this.comments = response.data.comments
				},
			)
		},

		getComments() {
			this.getData().then(() => this.comments = this.comments.reverse())
		},

		openAuthForm() {
			if (this.login) return

			const el = document
			const requestMethod = el.cancelFullscreen || el.webkitCancelFullScreen || el.mozCancelFullScreen || el.exitFullscreen || el.webkitExitFullscreen
			requestMethod.call(el)

			window.alsLoginUtils.openAuthForm()
		},

		getAuth() {
			let auth = window.alsLoginUtils.getAuth?.()

			auth ??= getCookie('als_login_sid')

			return auth
		},

		getNameStyle() {
			return {
				color: `white`,
			}
		},

		insertReply(name) {
			this.text += `@${name} `
			this.$refs.commentInput?.focus()
		},

		scrollToBottom() {
			const wrapper = this.$refs.commentsWrapper

			this.scrollTimeout = setTimeout(() => wrapper.scrollTo({behavior: 'smooth', top: 99999}), 3000)
		},

		cancelScroll() {
			clearTimeout(this.scrollTimeout)
			this.scrollTimeout = null
		},
	},
}
</script>
